#!/bin/sh
#
# Copyright (C) 2014 by The Art of Computer Technologies, Corp.
# All rights reserved.
#
#ident "@(#)sardo.sh       1.2h   2014.01.05 ACT. Katsumi.Nuruki"
#
# gather the system performance information.
#
#

#
# External environment variables:
#

# variable description:
# LOGDIR        This environment variable determines where sardo.sh data will
#               be gathered. $LOGDIR also use for the other sardo.sh functions.
#               The default value of $LOGDIR is "/var/tmp".
# SARDO         This environment variable is used for an instance identifier.
#               When the SARDO environment variable isn't defined, it will be
#               assumed three digits "001" which is defined same as "1".
#               Following sample is consequent of sardo directory name, if the
#               SARDO  environment variable is set to "2".
#
#                       "${LOGDIR}/sarstats_${HOSTNAME}_002"
#
# HOSTNAME      This environment variable is used for the hostname identifier.
#               When the HOSTNAME environment variable isn't defined, it will
#               be assumed `uname -n` output of this system.
#

#
# Update information.
# sardo.sh was created as first version.           ACT,Corp. 2003.10.28 Rev.1.0
# COR-001 Add exports variables.                   ACT,Corp. 2003.12.15 Rev.1.0a
# COR-002 Change termination proc.                 ACT,Corp. 2003.12.22 Rev.1.0a
# COR-003 Correct a tiny problem.                  ACT,Corp. 2003.12.22 Rev.1.0a
# COR-004 Change cmd compress to gzip.             ACT,Corp. 2004.02.12 Rev.1.0b
# COR-005 iostat -xd changed to -xdn.              ACT,Corp. 2004.03.18 Rev.1.0c
# COR-006 Add vmstat -s and uptime.                ACT,Corp. 2004.04.23 Rev.1.0d
# COR-007 The function name bug fix.               ACT,Corp. 2004.05.11 Rev.1.0e
# COR-008 Add kstat cmd for Solaris 8.             ACT,Corp. 2004.07.09 Rev.1.0f
# COR-009 Prevent pmap -x delaying.                ACT,Corp. 2005.04.06 Rev.1.0g
# COR-010 Change netstat -I format.                ACT,Corp. 2005.05.15 Rev.1.0h
# COR-011 Change eventcheck routine.               ACT,Corp. 2005.06.02 Rev.1.0j
# COR-012 Change uname -m to -i.                   ACT,Corp. 2005.06.05 Rev.1.0j
# COR-013 Check kstat & other patches.             ACT,Corp. 2005.06.05 Rev.1.0j
# COR-014 Add evck log & other patches.            ACT,Corp. 2005.06.07 Rev.1.0j
# COR-015 Change version number check.             ACT,Corp. 2005.06.26 Rev.1.0m
# COR-016 Check command path.                      ACT,Corp. 2005.08.12 Rev.1.0n
# COR-017 Change for Solaris 9.                    ACT,Corp. 2006.09.04 Rev.1.0p
# COR-018 Change for Solaris 10.                   ACT,Corp. 2006.11.11 Rev.1.0p
# COR-019 Add 5 sec interval vmstat.               ACT,Corp. 2006.11.12 Rev.1.1
# COR-020 Add uname -p.                            ACT,Corp. 2006.11.12 Rev.1.1
# COR-021 Stop pacct when "stop force".            ACT,Corp. 2006.11.12 Rev.1.1
# COR-022 Set EVENTCMD when noevent.               ACT,Corp. 2007.07.27 Rev.1.1a
# COR-023 Change for Solaris 11.                   ACT,Corp. 2009.09.30 Rev.1.2
# COR-024 Change UID variable name.                ACT,Corp. 2009.09.30 Rev.1.2
# COR-025 Change prtdiag path for i86pc.           ACT,Corp. 2009.10.14 Rev.1.2
# COR-026 Add mdb -k command ::memstat function.   ACT,Corp. 2010.02.13 Rev.1.2a
# COR-027 Check file size when estimate.           ACT,Corp. 2010.03.28 Rev.1.2b
# COR-028 Eliminate mdb error message.             ACT,Corp. 2010.06.21 Rev.1.2c
# COR-029 Not to exit failed devinf for zone.      ACT,Corp. 2010.06.21 Rev.1.2c
#

USRBIN=/usr/bin
USRLIBSA=/usr/lib/sa
USRLIBACCT=/usr/lib/acct
USRPROCBIN=/usr/proc/bin
USRSBIN=/usr/sbin
ETCDIR=/etc
PLATFORM="/usr/platform/`$USRBIN/uname -i`/sbin"			#COR-012

BASEINTERVAL=5
ONEDAYSECONDS=86400

# COR-025 start.
MISCCMD="\
$USRBIN/pagesize
$USRBIN/du -k
$USRBIN/uname -p
$USRBIN/uname -r
$USRBIN/hostid
$USRSBIN/sysdef
$PLATFORM/prtdiag -v
$USRBIN/uptime"								#COR-006

MISCI86CMD="\
$USRBIN/pagesize
$USRBIN/du -k
$USRBIN/uname -p
$USRBIN/uname -r
$USRBIN/hostid
$USRSBIN/sysdef
$USRSBIN/prtdiag -v
$USRBIN/uptime"								#COR-006
# COR-020 Added uname -p command.
# COR-025 end.

ONETIMECMD="\
$USRSBIN/crash
$USRBIN/df -k
$USRBIN/ipcs -a
$USRBIN/kstat
$USRBIN/mdb -k
$USRBIN/netstat -s
$USRBIN/netstat -k
$USRPROCBIN/pmap -x
$USRSBIN/swap -s
$USRBIN/vmstat -s"							#COR-006
# COR-008 Added kstat command.
# COR-026 Added mdb -k command for ::memstat.

# COR-005 An argument iostat has been changed from -xd to -xdn.
# COR-019 vmstat command has been deleted.
DURCMD="\
$USRBIN/vmstat -p
$USRBIN/iostat -xdn
$USRBIN/mpstat"

NDCMD="\
$USRBIN/ps -e -o user,pid,ppid,c,pri,nice,stime,etime,time,osz,rss,tty,args
$USRBIN/vmstat -i
$USRBIN/netstat -s
$USRBIN/netstat -an"

SADCCMD="\
$USRLIBSA/sadc"

GATHERFILES="\
$ETCDIR/hosts
$ETCDIR/system
$ETCDIR/vfstab
$USRSBIN/sar"

EVENTCHECK=sardo.event							#COR-011

intervaltimelist="\
3600
1800
900
600
60
30
10
5"

samplingtimelist="\
300
600
1800
3600
7200
10800
14400
18000
21600
25200
28800
32400
36000
39600
43200
46800
50400
54000
57600
61200
64800
68400
72000
75600
79200
82800
86400"

#
# Log records and display.
#
LogRecord() {
arg1="$1"
DATE=`$USRBIN/date '+%y%m%d%H%M%S'`
function=`$USRBIN/echo $FUNCTION |
$USRBIN/nawk '{ printf"%-6s\n",substr(toupper($1),1,6) }'`
$USRBIN/echo "$DATE $function :`$USRBIN/echo "$arg1"`" >> $LOGFILE
if [ $BATCHMODE = "NO" ]; then
	$USRBIN/echo "$arg1"
fi
}


#
# Set BATCHMODE environment variable.
#
SetBatchMode() {
if [ "${BATCHMODE}_x" = "_x" ]; then
	BATCHMODE=NO
else
	if [ $BATCHMODE != "YES" ] && [ $BATCHMODE != "NO" ]; then
		BATCHMODE=NO
	fi
fi
}


#
# Set the instance id and the other environment variables.
#
SetInstance() {
SET="NO"
if [ "${SARDO}_x" = "_x" ]; then
	INSTANCE="001"
else
	INSTANCE=`$USRBIN/echo $SARDO | $USRBIN/awk '{ printf"%03d\n",$1 }'`
fi
if [ "${HOSTNAME}_x" = "_x" ]; then
	HOSTNAME=`$USRBIN/uname -n`
fi
SARDODIR=$LOGDIR/sarstats_${HOSTNAME}_${INSTANCE}
LOGFILE=$SARDODIR/sardo.log
PARAMETER=$SARDODIR/sardo.param
TEMPFILE=$SARDODIR/tempfile$$
WORKFILE=$SARDODIR/workfile$$
WORK=$LOGDIR/workfile$$
SARDO=$INSTANCE								#COR-001
export SARDO LOGDIR SARDODIR LOGFILE PARAMETER				#COR-001
# COR-018 start.
OSVERSION=`$USRBIN/uname -r  | $USRBIN/awk -F"." '{ printf("%d%02d\n",$1,$2) }'`
Solaris8=`$USRBIN/echo 5.8   | $USRBIN/awk -F"." '{ printf("%d%02d\n",$1,$2) }'`
Solaris9=`$USRBIN/echo 5.9   | $USRBIN/awk -F"." '{ printf("%d%02d\n",$1,$2) }'`
Solaris10=`$USRBIN/echo 5.10 | $USRBIN/awk -F"." '{ printf("%d%02d\n",$1,$2) }'`
# COR-023 start.
Solaris11=`$USRBIN/echo 5.11 | $USRBIN/awk -F"." '{ printf("%d%02d\n",$1,$2) }'`
export OSVERSION Solaris8 Solaris9 Solaris10 Solaris11
# COR-023 end.
# COR-018 end.
}


#
# Check root uid.
#
UidCheck() {
userUID=`$USRBIN/id |							#COR-024
$USRBIN/awk '{	split($1,array,"=")
		split(array[2],id,"(")
		if (( id[1] == 0 ) && ( substr(id[2],1,4) == "root" ))
			result="OK"
		else
			result="NG"
		}
END {	print result }'`
if [ $userUID != "OK" ]; then						#COR-024
	$USRBIN/echo "ERROR: uid is not root."
	ClearfAndExit 1
fi
}


#
# get the command path instead of which(1) command.
#
whichcmd () {
mod=$1
if [ "${mod}_x" = "_x" ]; then
	LogRecord "ERROR: whichcmd argument missing."
	ClearfAndExit 1
fi
COMMANDPATH=NOTFOUND
found=NO
while [ $found = "NO" ]
do
	for i in `$USRBIN/echo $PATH | $USRBIN/awk '{
		n = split($0,array,":")
		for ( i = 1 ; i <= n ; i++ )
			printf"%s\n",array[i]
		printf"\n" }'`
	do
		if [ -x ${i}/${mod} ]  && [ "${i}" != "." ]; then
			COMMANDPATH=${i}/${mod}
			found=YES
			break
		fi
	done
	found=$COMMANDPATH
done
}


#
# Check and make sardo directory.
#
SarDoDirectory() {
if [ ! -d $SARDODIR ]; then
	if [ $FUNCTION != "unpack" ] && [ $FUNCTION != "estimate" ]; then
		$USRBIN/mkdir $SARDODIR
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: can't make sardo directory."
			ClearfAndExit 1
		fi
	fi
fi
}


#
# Print "."(dot) without LF.
#
PrintDot() {
if [ $BATCHMODE = "NO" ]; then
	$USRBIN/echo ".\c"
fi
}


#
# Feed one line.
#
BlankLine() {
if [ $BATCHMODE = "NO" ]; then
	$USRBIN/echo ""
fi
}


#
# Setup the parameter file.
#
spsetup() {
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
case $numberofargs in
4)	SetTimeParameters $ARG2 $ARG3 $ARG4
	$USRBIN/echo "$INTERVAL $STIME $ETIME" > $PARAMETER
	RETVAL=$?					;;
*)	RETVAL=1					;;
esac
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: spsetup parameter setting was failed."
	ClearfAndExit 1
fi
LogRecord "INFO:  spsetup was successfully completed."
}


#
# Execute one time commands.
#
DoCommands() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$USRBIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $USRBIN/awk '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$USRBIN/cat $TEMPFILE |
	$USRBIN/awk '{	if ( NR == '"$N"' )
				print $0 }'`
	BASENAME=`$USRBIN/echo "$CMD" | $USRBIN/awk '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$USRBIN/basename $BASENAME`
	SUFFICS=`$USRBIN/echo "$CMD" |
	$USRBIN/awk '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	DATE=`$USRBIN/date '+%H:%M:%S'`
	OFILE=`$USRBIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
	$USRBIN/sed 's/__/_/g'`
	$USRBIN/echo "${BASENAME}_${SUFFICS} $DATE" > $OFILE
	if [ ! -x $DIRNAME/$BASENAME ]; then
		# COR-013 start.
		if [ $OSVERSION -lt $Solaris8 ] && [ $BASENAME = "kstat" ]; then
			LogRecord "NOTE:  $DIRNAME/$BASENAME isn't executable or doesn't exist."
		else
			# COR-017 start.
			if [ $BASENAME = "crash" ]; then
				if [ $ADBMODIFIER = "X" ]; then
					MODIFIER="D"
				else
					MODIFIER="E"
				fi
				if [ $OSVERSION -eq $Solaris10 ] || [ $OSVERSION -eq $Solaris11 ]; then	#COR-023
					$USRBIN/mdb -k -y > $OFILE 2>/dev/null <<-EOF			#COR-028
					::kmastat
					physinstalled/$MODIFIER
					physmem/$MODIFIER
					::quit
					EOF
					RETVAL=$?
				else
					$USRBIN/adb -k 1>$OFILE 2>&1 <<-EOF
					physinstalled/$MODIFIER
					physmem/$MODIFIER
					EOF
					RETVAL=$?
				fi
				if [ $RETVAL -ne 0 ]; then
					LogRecord "ERROR: ${BASENAME}[adb] -${SUFFICS} return value wasn't 0"
				else
					LogRecord "INFO:  ${BASENAME}[adb] -${SUFFICS} was successfully completed."
				fi
			else
				LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable or doesn't exist."
			fi
			# COR-017 end.
		fi
		# COR-013 end.
	else
		# COR-018 start.
		if [ $OSVERSION -eq $Solaris10 ] && [ $BASENAME = "netstat" ] && [ $SUFFICS = "k" ]; then
			LogRecord "INFO:  $BASENAME -${SUFFICS} was skipped."
		else
		    # COR-023 start.
		    if [ $OSVERSION -eq $Solaris11 ] && [ $BASENAME = "netstat" ] && [ $SUFFICS = "k" ]; then
			LogRecord "INFO:  $BASENAME -${SUFFICS} was skipped."
		    else
		    # COR-023 end.
			case $BASENAME in
			crash)	if [ $OSVERSION -eq $Solaris10 ] || [ $OSVERSION -eq $Solaris11 ]; then	#COR-023
					if [ $ADBMODIFIER = "X" ]; then
						MODIFIER="D"
					else
						MODIFIER="E"
					fi
					$USRBIN/mdb -k -y > $OFILE 2>/dev/null <<-EOF			#COR-028
					::kmastat
					physinstalled/$MODIFIER
					physmem/$MODIFIER
					::quit
					EOF
					RETVAL=$?
				else
					$DIRNAME/$BASENAME 1>/dev/null 2>&1 <<-EOF
					kmastat -w $OFILE
					od -w $OFILE -d physinstalled
					q
					EOF
					RETVAL=$?
				fi			;;
			pmap)	# COR-009 begin.
				if [ $ID = "begin" ]; then
					for pid in `$USRBIN/ps -e -o pid=`
					do
						$DIRNAME/$BASENAME -x $pid 2>/dev/null
					done >> $OFILE &
				else
					for pid in `$USRBIN/ps -e -o pid=`
					do
						$DIRNAME/$BASENAME -x $pid 2>/dev/null
					done >> $OFILE
				fi
				# COR-009 end.
				RETVAL=0		;;
			kstat)	# COR-013 start.
				if [ $OSVERSION -ge $Solaris8 ]; then
					$CMD >> $OFILE
					RETVAL=$?
				fi			;;
				# COR-013 end.
			mdb)	# COR-026 begin.
				OFILE=`$USRBIN/echo $SARDODIR/${ID}/${BASENAME}_memstat_info |
				$USRBIN/sed 's/__/_/g'`
				if [ $OSVERSION -eq $Solaris10 ] || [ $OSVERSION -eq $Solaris11 ]; then
					$USRBIN/mdb -k -y > $OFILE 2>/dev/null <<-EOF		#COR-028
					::memstat
					::quit
					EOF
					RETVAL=$?
				fi			;;
				# COR-026 end.
			*)	$CMD >> $OFILE
				RETVAL=$?		;;
			esac
			if [ $RETVAL -ne 0 ]; then
				LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
			else
				LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully completed."
			fi
		    fi #COR-023
		fi
		# COR-018 end.
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the duration type commands.
#
DoDurationCmds() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$USRBIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi

# COR-010 start.
for i in `$USRSBIN/ifconfig -a |
	$USRBIN/awk '{	ni=split($1,interface,":")
			nf=split($2,array,"=")
			if (( nf >= 2 ) && ( ni >= 2 ) && ( array[1] == "flags" ))
				printf("%s\n",interface[1]) }'`
#for i in `$USRBIN/netstat -i |
#	$USRBIN/awk '{	if ( $1 == "lo0" ) {
#					prflag="ON"
#				}
#				if (( prflag == "ON" ) && ( NF != 0))
#					print $1 }'`
# COR-010 end.
do
	$USRBIN/echo "$USRBIN/netstat -I $i" >> $WORKFILE
done
$USRBIN/awk '{	if ( NF != 0 ) print $0 }' $TEMPFILE >> $WORKFILE
$USRBIN/mv $WORKFILE $TEMPFILE
DOCOUNT=`$USRBIN/expr $COUNT + 1`
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $USRBIN/awk '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$USRBIN/cat $TEMPFILE |
	$USRBIN/awk '{	if ( NR == '"$N"' )
				print $0 }'`
	IF=`$USRBIN/echo "$CMD" | $USRBIN/awk '{ print $3 }'`
	BASENAME=`$USRBIN/echo "$CMD" | $USRBIN/awk '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$USRBIN/basename $BASENAME`
	SUFFICS=`$USRBIN/echo "$CMD" |
	$USRBIN/awk '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	if [ $BASENAME = "netstat" ]; then
		OFILE=`$USRBIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_${IF}_info |
		$USRBIN/sed 's/__/_/g'`
	else
		OFILE=`$USRBIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
		$USRBIN/sed 's/__/_/g'`
	fi
	if [ ! -x $DIRNAME/$BASENAME ]; then
		LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable or doesn't exist." # COR-013
	else
		# COR-010 start.
		if [ $OSVERSION -lt $Solaris8 ] && [ $BASENAME = "netstat" ]; then
			DOCOUNT=""
		fi
		# COR-010 end.

		case $BASENAME in
		*)		$CMD $INTERVAL $DOCOUNT > $OFILE &
				PID=$!
				RETVAL=$?		;;
		esac
		if [ $RETVAL -ne 0 ]; then
			LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
		else
			LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully scheduled."
		fi
		$USRBIN/echo $PID >> $SARDODIR/pidlist
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the non duration type commands.
#
DoNonDurationCmds() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$USRBIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
$USRBIN/echo "`$USRBIN/date '+%H:%M:%S'`" > $SARDODIR/go
CMDCOUNT=`$USRBIN/wc -l $TEMPFILE | $USRBIN/awk '{ print $1 }'`
N=1
while [ $N -le $CMDCOUNT ]
do
	CMD=`$USRBIN/cat $TEMPFILE |
	$USRBIN/awk '{	if ( NR == '"$N"' )
				print $0 }'`
	BASENAME=`$USRBIN/echo "$CMD" | $USRBIN/awk '{ print $1 }'`
	DIRNAME=`$USRBIN/dirname $BASENAME`
	BASENAME=`$USRBIN/basename $BASENAME`
	SUFFICS=`$USRBIN/echo "$CMD" |
	$USRBIN/awk '{ if ( NF == 2 ) {
				split($2,array,"-")
				name=array[2]
				}
			else
				name=""
			}
	END { print name }'`
	DATE=`$USRBIN/date '+%H:%M:%S'`
	OFILE=`$USRBIN/echo $SARDODIR/${ID}/${BASENAME}_${SUFFICS}_info |
	$USRBIN/sed 's/__/_/g'`
	if [ ! -x $DIRNAME/$BASENAME ]; then
		LogRecord "ERROR: $DIRNAME/$BASENAME isn't executable or doesn't exist." # COR-013
	else
		while [ -f $SARDODIR/go ]
		do
			$USRBIN/echo "${BASENAME}_${SUFFICS} `$USRBIN/date '+%H:%M:%S'`" >> $OFILE
			$CMD >> $OFILE &				#COR-014
			$USRBIN/sleep $INTERVAL
		done &
		PID=$!
		RETVAL=$?
		if [ $RETVAL -ne 0 ]; then
			LogRecord "ERROR: $BASENAME -${SUFFICS} return value wasn't 0"
		else
			LogRecord "INFO:  $BASENAME -${SUFFICS} was successfully scheduled."
		fi
		$USRBIN/echo $PID >> $SARDODIR/pidlist
	fi
	N=`$USRBIN/expr $N + 1`
done
}


#
# Execute the sadc commands.
#
DoSaDcCommand() {
ID=$1
if [ ! -d $SARDODIR/${ID} ]; then
	$USRBIN/mkdir $SARDODIR/${ID}
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: can't make $SARDODIR/${ID} direcotry."
		ClearfAndExit 1
	fi
fi
CMD=`$USRBIN/cat $TEMPFILE`
if [ ! -x $CMD ]; then
	LogRecord "ERROR: $CMD isn't executable or doesn't exist."	#COR-013
	ClearfAndExit 1							#COR-007
fi
SADCCOUNT=`$USRBIN/echo $COUNT |
$USRBIN/awk '{	count=$1*'"$INTERVAL"'/'"$BASEINTERVAL"'
		count++ }
END { print count }'`
if [ -f $SARDODIR/${ID}/sadc_info ]; then
	$USRBIN/rm $SARDODIR/${ID}/sadc_info
fi
$CMD $BASEINTERVAL $SADCCOUNT $SARDODIR/${ID}/sadc_info &
PID=$!
RETVAL=$?
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: sadc command return value wasn't 0"
else
	LogRecord "INFO:  sadc command was successfully scheduled."
fi
$USRBIN/echo $PID >> $SARDODIR/pidlist

# COR-019 start.
if [ -f $SARDODIR/data/vmstat_info ]; then
	$USRBIN/rm $SARDODIR/data/vmstat_info
fi
$USRBIN/vmstat $BASEINTERVAL $SADCCOUNT > $SARDODIR/data/vmstat_info &
PID=$!
RETVAL=$?
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: vmstat command return value wasn't 0"
else
	LogRecord "INFO:  vmstat command was successfully scheduled."
fi
$USRBIN/echo $PID >> $SARDODIR/pidlist
# COR-019 end.

}


#
# Check process account is running or not.
#
CheckPacct() {
if [ -x $USRBIN/isainfo ]; then
	CPUBITS=`$USRBIN/isainfo -b`
else
	CPUBITS=32
fi
case $CPUBITS in
32)	ADBMODIFIER=X	;;
64)	ADBMODIFIER=J	;;
esac
export ADBMODIFIER							#COR-017

# COR-018 start.
if [ $OSVERSION -le $Solaris9 ]; then
	PACCT=`$USRBIN/adb -k > $TEMPFILE <<-EOF
	acctvp/$ADBMODIFIER
	\$q
	EOF
	$USRBIN/awk '{	if (( $1 == "acctvp:" ) && ( NF == 2 ))
				if ( $2 != 0 ) {
					pacct="DONTRUN"
					break
				}
			}
	END {	if ( pacct == "DONTRUN" )
			print pacct
		else
			print "RUN"
	}' $TEMPFILE`
else
	PACCT=`$USRSBIN/modinfo |
	$USRBIN/awk 'BEGIN { flag="NOCHECK" }
	{
	if ( $6 == "sysacct" ) {
		flag="VNODECHECK"
		exit
		}
	} END { printf("%s\n",flag) }'`
	if [ $PACCT = "NOCHECK" ]; then
		PACCT="RUN"
	else
		if [ $ADBMODIFIER = "J" ]; then
			adbparam="(*(acct_list+0x10)-8)/1J"
		else
			adbparam="(*(acct_list+0x08)-4)/1X"
		fi
		PACCT=`$USRBIN/mdb -k > $TEMPFILE 2>/dev/null <<-EOF	#COR-028
		$adbparam
		\$q
		EOF
		$USRBIN/awk '{	if (( substr($1,1,2) == "0x" ) && ( NF == 2 ))
					if ( $2 != 0 ) {
						pacct="DONTRUN"
						break
					}
				}
		END {	if ( pacct == "DONTRUN" )
				print pacct
			else
				print "RUN"
		}' $TEMPFILE`
	fi
fi
if [ -s $TEMPFILE ]; then
	$USRBIN/rm $TEMPFILE
fi
# COR-018 end.
}


#
# Clear files and terminating with arg1.
#
ClearfAndExit() {
RETVAL=$1
for i in $TEMPFILE $WORKFILE $WORK
do
	if [ -f $i ]; then
		$USRBIN/rm $i
	fi
done
exit $RETVAL
}


#
# Convert the time string to the integer in seconds.
#
S2I() {
TIMEVALUE=`$USRBIN/echo $1 |
$USRBIN/awk '{ n=split($1,array,":") }
END {	if ( n > 1 ) {
		if (( array[1] < 24 ) && ( array[1] >= 0 ))
			hsec=array[1]*3600
		else {
			if ( array[1] == 24 ) {
				printf"%d\n",array[1]*3600
				exit
			}
			printf"INVALID\n"
			exit
		}
		if (( array[2] < 60 ) && ( array[2] >= 0 ))
			msec=array[2]*60
		else {
			printf"INVALID\n"
			exit
		}
		if ( n == 3 ) {
			if (( array[3] < 60 ) && ( array[3] >= 0 ))
				sec=array[3]
			else {
				printf"INVALID\n"
				exit
			}
		}
		else
			sec=0
		printf"%d\n",hsec+msec+sec
		}
	else
		printf"INVALID\n" }'`
if [ $TIMEVALUE = "INVALID" ]; then
	LogRecord "ERROR: the time format invalid."
	ClearfAndExit 1
fi
}


#
# Convert the integer second to the time string.
#
I2S() {
TIMESTRING=`$USRBIN/echo $1 |
$USRBIN/awk '{	h=$1/3600
		r=$1%3600
		m=r/60
		s=r%60
		if ( h >= 24 )
			h=h-24
		printf"%02d:%02d:%02d\n",h,m,s }'`
}


#
# Set the times environment variables.
#
SetTimeParameters() {
arg1=$1
arg2=$2
arg3=$3
#
# $arg1         : sampling interval xx in seconds, xxmin, or xxsec.
# $arg2         : start time now or HH:MM[:SS] format.
# $arg3         : end time xxmin, xxhour, or HH:MM[:SS] format.
#
# This function sets times environment variables as below.
# $SAMPLING     : sampling mode. INTHEDAY   = $arg2 < $arg3
#                                OVERTHEDAY = $arg2 > $arg3
# $INTERVAL	: sampling interval in seconds.
# $COUNT	: sampling count.
# $ctime        : current time in seconds.
# $CTIME        : current time with HH:MM:SS format.
# $stime	: start time in seconds.
# $STIME	: start time with HH:MM:SS format.
# $etime	: end time in seconds.
# $ETIME	: end time with HH:MM:SS format.
#
#
INTERVAL=`$USRBIN/echo $arg1 |
$USRBIN/awk '{
if ( substr($1,length($1)-3+1,3) == "min" ) {
	split($1,array,"min")
	if (( array[1] <= 60 ) && ( array[1] >= 1 ))
		n=array[1]*60
	else
		n="NG"
	}
else
	if ( substr($1,length($1)-3+1,3) == "sec" ) {
		split($1,array,"sec")
		if (( array[1] <= 3600 ) && ( array[1] >= 5))
			n=array[1]
		else
			n="NG"
		}
	else
		if (( $1 <= 3600 ) && ( $1 >= 5 ))
			n=$1
		else
			n="NG"
	}
END { printf"%d\n",n }'`
if [ $INTERVAL = "NG" ]; then
	LogRecord "ERROR: the sampling interval invalid."
	ClearfAndExit 1
fi
CTIME=`$USRBIN/date '+%H:%M:%S'`
S2I $CTIME
ctime=$TIMEVALUE
if [ $arg2 = "now" ]; then
	STIME=$CTIME
else
	S2I $arg2
	I2S $TIMEVALUE
	STIME=$TIMESTRING
fi
S2I $STIME
stime=$TIMEVALUE
$USRBIN/echo $arg3 | $USRBIN/grep ":" 1>/dev/null
if [ $? -eq 0 ]; then
	S2I $arg3
	etime=$TIMEVALUE
else
	RETVAL=`$USRBIN/echo $arg3 |
	$USRBIN/awk '{
	if ( substr($1,length($1)-3+1,3) == "min" ) {
		split($1,array,"min")
		if (( array[1] <= 1440 ) && ( array[1] >=1 ))
			n=array[1]*60
		else
			n="NG"
		}
	else
		if ( substr($1,length($1)-4+1,4 ) == "hour" ) {
			split($1,array,"hour")
			if (( array[1] <= 24 ) && ( array[1] >=1 ))
				n=array[1]*3600
			else
				n="NG"
			}
		else
			n=$1
		}
	END { printf"%s\n",n }'`
	if [ $RETVAL = "NG" ]; then
		LogRecord "ERROR: the end parameter is invalid."
		ClearfAndExit 1
	fi
	etime=`$USRBIN/expr $RETVAL + $stime`
fi
TIMEDIFF=`$USRBIN/expr $etime - $stime`
if [ $TIMEDIFF -eq $ONEDAYSECONDS ]; then
	etime=`$USRBIN/expr $etime - 1`
fi
I2S $etime
ETIME=$TIMESTRING
if [ $stime -eq $etime ]; then
	LogRecord "ERROR: the start and the end time are identical."
	ClearfAndExit 1
fi
if [ $stime -gt $etime ]; then
	etime=`$USRBIN/echo $etime |
	$USRBIN/awk '{ printf"%d\n",'"$ONEDAYSECONDS"' + $1 }'`
	SAMPLING=OVERTHEDAY
else
	SAMPLING=INTHEDAY
fi
COUNT=`$USRBIN/echo $INTERVAL |
$USRBIN/awk '{ printf"%d\n",('"$etime"'-'"$stime"')/$1 }'`
if [ $COUNT -eq 0 ]; then
	LogRecord "ERROR: the time parameters are inconsistent."
	ClearfAndExit 1
fi
}


#
# Test the sadc process is running for start process.
#
SadcExecTest() {
SADCCHECK=NO
SADCCHECKCOUNT=5
N=1
while [ $SADCCHECK != "YES" ]
do
	$USRBIN/ps -e -o pid,args |
	$USRBIN/awk '{
	if ( $2 == "/usr/lib/sa/sadc" ) {
		found="YES"
		printf"%d\n",$1
		}
	}
	END {	if ( found != "YES" )
			printf"NOTFOUND\n"
	}' | $USRBIN/sort -u > ${SARDODIR}/sadcpidlist
	for i in `$USRBIN/cat ${SARDODIR}/sadcpidlist`
	do
		SADCPID=$i
		if [ $SADCPID != "NOTFOUND" ] || [ $SADCPID -eq 0 ]; then
			RETVAL=`$USRBIN/awk '{
			if ( $1 == '"$SADCPID"' ) {
				found="YES"
				exit
				}
			}
			END {	if ( found == "YES" )
					printf"OK\n"
				else
					printf"NG\n"
			}' $SARDODIR/pidlist'`
		fi
		if [ $RETVAL = "OK" ]; then
			SADCCHECK=YES
			break
		fi
	done
	N=`$USRBIN/expr $N + 1`
	if [ $N -gt $SADCCHECKCOUNT ]; then
		RETVAL=NG
		SADCCHECK=YES
	else
		$USRBIN/sleep 1
	fi
done
if [ -f ${SARDODIR}/sadcpidlist ]; then
	$USRBIN/rm ${SARDODIR}/sadcpidlist
fi
}


#
# The sardo.sh termination process.
#
SarDoTermProc() {
SADCPID=$1
SEQUENCE=START
while [ -f ${SARDODIR}/go ]
do
	LogRecord "INFO:  wait for the end time [ $ETIME ] to finish."
	if [ $SAMPLING = "INTHEDAY" ]; then
		REMAIN=`$USRBIN/expr $etime - $ctime`
	else
		REMAIN=`$USRBIN/echo $ctime | $USRBIN/awk '{
		todaysec = '"$ONEDAYSECONDS"' - $1
		nextdaysec = '"$etime"' - '"$ONEDAYSECONDS"'
		sec = todaysec + nextdaysec }
		END { printf"%d\n",sec }'`
	fi
	if [ $stime -gt $ctime ]; then
		waitsec=`$USRBIN/expr $stime - $ctime`
	else
		waitsec=0
	fi

	# COR-002 start.
	if [ $SEQUENCE = "START" ]; then
		REMAIN=`$USRBIN/expr $REMAIN - $waitsec`
	else
		REMAIN=$BASEINTERVAL
	fi
	BATCHMODE=YES
	LogRecord "INFO:  remaining time is $REMAIN seconds."
	$USRBIN/sleep $REMAIN
	# COR-002 end.

	SEQUENCE=STOP
	SADC=`$USRBIN/ps -e -o pid,args |
	$USRBIN/awk '{
	if (( $2 == "/usr/lib/sa/sadc" ) && ( $1 == '"$SADCPID"' )) {
		found="YES"
		pid=$1
		exit
		}
	}
	END {	if ( found == "YES" )
			printf"ALIVE\n"
		else
			printf"NOTALIVE\n"
	}'`
	if [ $SADC = "NOTALIVE" ]; then
		$SARDOEXECSHELL stop
		if [ -f $SARDODIR/termprocpid ]; then
			$USRBIN/rm $SARDODIR/termprocpid
		fi
		cd ${LOGDIR}
		while [ -f $SARDODIR/pidlist ]
		do
			$USRBIN/sleep 1
		done
		$SARDOEXECSHELL tar 1>/dev/null 2>&1
		ClearfAndExit
	fi
done 2>/dev/null &
PID=$!
RETVAL=$?
export PID RETVAL
$USRBIN/echo $PID > $SARDODIR/termprocpid
}


#
# spautoexec: start sardo automatically.
#
spautoexec() {
#
BATCHMODE=YES; export BATCHMODE
#
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
SetTimeParameters $1 $2 $3
export INTERVAL COUNT ctime CTIME stime STIME etime ETIME SAMPLING
$SARDOEXECSHELL setup $INTERVAL $STIME $ETIME
if [ $? -ne 0 ]; then
	LogRecord "ERROR: spautoexec setup function terminated."
	ClearfAndExit 1
fi
$SARDOEXECSHELL start
if [ $? -ne 0 ]; then
	LogRecord "ERROR: spautoexec start function terminated."
	ClearfAndExit 1
fi
}


#
# Search mount point.
#
SearchMountPoint() {
DIRECTORY=$1
if [ $GETINFO = "THISHOST" ]; then
	$USRBIN/df -k > $WORK
else
	$USRBIN/cp $ESDIR/sarstats_${ESHOST}_${ESID}/end/df_k_info $WORK
fi
MOUNTPOINT=NO
MOUNT=$DIRECTORY
while [ $MOUNTPOINT = "NO" ]
do
	RESULT=`$USRBIN/echo $MOUNT |
	$USRBIN/awk '{ n=split($1,array,"/") }
	END {	if ( n > 2 )
			printf"%s\n","'"$MOUNT"'"
		else
			if ( n == 2 )
				printf"THIS\n"
			else
				printf"YES\n" }'`
	if [ $RESULT = "YES" ]; then
		MOUNTPOINT="YES"
		break
	else
		if [ $RESULT = "THIS" ]; then
			MOUNTPOINT="YES"
			break
		else
			MOUNT=$RESULT
		fi
	fi
	MOUNT=`$USRBIN/dirname $MOUNT`
done
if [ $GETINFO = "THISHOST" ]; then
	VFSTAB=$ETCDIR/vfstab
else
	VFSTAB=$ESDIR/sarstats_${ESHOST}_${ESID}/misc/vfstab
fi
MOUNTCOUNT=`$USRBIN/cat $VFSTAB | $USRBIN/grep -v "#" |
$USRBIN/wc -l | $USRBIN/awk '{ printf"%d\n",$1 }'`
N=1
DIR=$MOUNT
MOUNTPOINT=NOTFOUND
while [ $N != $MOUNTCOUNT ]
do
	for i in `$USRBIN/cat $VFSTAB | $USRBIN/grep -v "#" |
		  $USRBIN/awk '{ printf"%d\n",$3 }'`
	do
		RESULT=`
		$USRBIN/awk '{
		if ( "'"$DIR"'" == $6 ) {
			found="YES"
			name=$6
			exit
			}
		}
		END {	if ( found == "YES" )
				printf"%s\n",name
			else
				printf"NOTFOUND\n"
		}' $WORK`
		if [ $RESULT != "NOTFOUND" ]; then
			MOUNTPOINT=$RESULT
			break
		fi
		N=`$USRBIN/expr $N + 1`
	done
	N=$MOUNTCOUNT
done
if [ $MOUNTPOINT = "NOTFOUND" ]; then
	MOUNTPOINT="/"
fi

AVAILBLOCK=`$USRBIN/awk {'
if ( $6 == "'"$MOUNTPOINT"'" )
	printf"%d\n", $2 * 0.9 - $3
}' $WORK`
$USRBIN/rm $WORK
}


#
# Estimate how many data the sardo.sh can gather into the LOGDIR directory.
#
spestimate() {
ESHOST=$1
ESID=$2
ESDIR=$3
ARGS=$#
if [ $ARGS -ne 3 ] && [ $ARGS -ne 0 ] ; then
	$USRBIN/echo "ERROR: spestimate invalid param."
	exit 1
fi
if [ "${ESHOST}_x" = "_x" ]; then
	ESHOST=`$USRBIN/uname -n`
	GETINFO=THISHOST
else
	HOSTNAME=`$USRBIN/uname -n`
	if [ $HOSTNAME = $ESHOST ]; then
		GETINFO=THISHOST
	else
		GETINFO=FROMFILES
	fi
fi
if [ "${ESID}_x" = "_x" ]; then
	if [ "${SARDO}_x" = "_x" ]; then
		ESID=1
	else
		ESID=$SARDO
	fi
fi

ESID=`$USRBIN/echo $ESID | $USRBIN/awk '{ printf"%03d\n",$1 }'`		#COR-003

if [ "${ESDIR}_x" = "_x" ]; then
	ESDIR=$LOGDIR
fi

if [ ! -d ${ESDIR} ]; then
	$USRBIN/echo "ERROR: directory ${ESDIR} was not found."
	exit 1
fi

if [ -d ${ESDIR}/sarstats_${ESHOST}_${ESID} ]; then
	$USRBIN/echo "ERROR: now sardo_${ESHOST}_${ESID} directory is active."
	exit 1
fi


cd $ESDIR

FILECOUNT=`$USRBIN/ls |
$USRBIN/grep sarstats_${ESHOST}_${ESID}_.............tar.gz |		#COR-004
$USRBIN/wc -l | $USRBIN/awk '{ printf"%d\n",$1 }'`

ARCHIVEFILE=`$USRBIN/ls |
$USRBIN/grep sarstats_${ESHOST}_${ESID}_.............tar.gz`		#COR-004

if [ $FILECOUNT -gt 1 ]; then
	$USRBIN/echo "ERROR: several sardo archive files were found."
	exit 1
fi

if [ "${ARCHIVEFILE}_x" = "_x" ]; then
	$USRBIN/echo "ERROR: no any sardo archive files were found."
	exit 1
fi

# COR-027 start.
ARCHIVESIZE=0
while [ $ARCHIVESIZE -eq 0 ]
do
	ARCHIVESIZE=`$USRBIN/du -k $ARCHIVEFILE  | $USRBIN/awk '{ printf"%d\n",$1 }'`
	$USRBIN/sleep 3
done
# COR-027 end.

$USRBIN/gunzip -d $ARCHIVEFILE						#COR-004
if [ $? -ne 0 ]; then
	$USRBIN/echo "ERROR: gunzip command was failed."		#COR-004
	exit 1
fi

TARFILENAME=`$USRBIN/echo $ARCHIVEFILE | $USRBIN/awk '{ split($0,array,".") }
END {	printf"%s.tar\n",array[1] }'`

# COR-027 start.
TARFILESIZE=0
while [ $TARFILESIZE -eq 0 ]
do
	TARFILESIZE=`$USRBIN/du -k $TARFILENAME | $USRBIN/awk '{ printf("%d\n",$1) }'`
	$USRBIN/sleep 3
done
# COR-027 end.

$USRBIN/tar -xvf $TARFILENAME 1>/dev/null 2>&1
if [ $? -ne 0 ]; then
	$USRBIN/echo "ERROR: tar command was failed."
	exit 1
fi

cd $ESDIR/sarstats_${ESHOST}_${ESID}

CHECKCOUNT=12
N=1
EXIST=YES
while [ $EXIST = "YES" ]
do
	if [ ! -f sardosize_info ]; then
		$USRBIN/echo "\c" > sardosize_info
		EXIST=NO
		break
	fi
	if [ $N -lt $CHECKCOUNT ]; then
		$USRBIN/sleep $BASEINTERVAL
	else
		$USRBIN/echo "ERROR: sardosize_info file exists."
		cd ..
		$USRBIN/gzip $TARFILENAME				#COR-004
		if [ $? -ne 0 ]; then
			$USRBIN/echo "ERROR: gzip command was failed."	#COR-004
			$USRBIN/rm $WORK
			exit 1
		fi
		$USRBIN/rm -rf sarstats_${ESHOST}_${ESID}
		$USRBIN/rm $WORK
		exit 1
	fi
	N=`$USRBIN/expr $N + 1`
done

$USRBIN/du -k ../sarstats_${ESHOST}_${ESID} > $WORK
if [ $? -ne 0 ]; then
	$USRBIN/echo "ERROR: du command was failed."
	$USRBIN/rm $WORK
	exit 1
fi

SARDODIRSIZE=`$USRBIN/tail -1 $WORK | $USRBIN/awk '{ printf"%d\n",$1}'`

$USRBIN/grep "sarstats_${ESHOST}_${ESID}/pacct" $WORK 1>/dev/null 2>&1
if [ $? -eq 0 ]; then
	PACCOUNTSIZE=`$USRBIN/grep "sarstats_${ESHOST}_${ESID}/pacct" $WORK |
	$USRBIN/awk '{ printf"%d\n",$1 }'`
else
	PACCOUNTSIZE=0
fi

for i in begin end misc
do
	ONETIMETOTALSIZE=`$USRBIN/grep $i $WORK |
	$USRBIN/awk '{ n = '"$ONETIMETOTALSIZE"' + $1 } END { printf"%d\n",n }'`
done

DATADIRSIZE=`$USRBIN/grep data $WORK | $USRBIN/awk '{ printf"%d\n",$1 }'`
SADCDIRSIZE=`$USRBIN/grep sadc $WORK | $USRBIN/awk '{ printf"%d\n",$1 }'`

PMAPFILECOUNT=`$USRBIN/grep pmap $WORK 1>/dev/null 2>&1
		if [ $? -ne 0 ]; then
			$USRBIN/echo 0
		else
			if [ -d pmap ]; then
				$USRBIN/ls pmap |
				$USRBIN/wc -l | $USRBIN/awk '{ printf"%d\n",$1 }'
			else
				$USRBIN/echo 0
			fi
		fi`
if [ $PMAPFILECOUNT -ne 0 ]; then
	PMAPDIRSIZE=`$USRBIN/grep pmap $WORK |
	$USRBIN/awk '{ printf"%d\n",$1 }'`
else
	PMAPDIRSIZE=0
fi

INTERVAL=`$USRBIN/awk '{
printf"%d\n",$1 }' sardo.param`

STIME=`$USRBIN/awk '{ printf"%s\n",$2 }' sardo.param`
S2I $STIME
stime=$TIMEVALUE

ETIME=`$USRBIN/awk '{ printf"%s\n",$3 }' sardo.param`
S2I $ETIME
etime=$TIMEVALUE

if [ $stime -gt $etime ]; then
	etime=`$USRBIN/expr $etime + $ONEDAYSECONDS`
fi

SAMPLINGTIME=`$USRBIN/expr $etime - $stime`
SAMPLINGCOUNT=`$USRBIN/expr $SAMPLINGTIME \/ $INTERVAL`

SearchMountPoint $ESDIR

$USRBIN/echo "\
hostname           : `$USRBIN/echo ${ESHOST}`
hostid             : `$USRBIN/awk '{	if ( $1 == "hostid_" ) {
						getline
						id=$1
					}
				}
		      END { printf"%s\n",id }' misc/hostid_info`
date               : `$USRBIN/cat date.info`

unitvalue          : 1024 bytes (kiro-bytes) 1 block unit

archivefilecount   : $FILECOUNT
availableblock     : $AVAILBLOCK
datadirectorysize  : $DATADIRSIZE
logdirectory       : $ESDIR
onetimetotalsize   : $ONETIMETOTALSIZE
paccountsize       : $PACCOUNTSIZE
packedarchivesize  : $ARCHIVESIZE
pmapdirectorysize  : $PMAPDIRSIZE
pmapfilecount      : $PMAPFILECOUNT
sadcdirectorysize  : $SADCDIRSIZE
samplinginterval   : $INTERVAL
samplingtime       : $SAMPLINGTIME
samplingcount      : $SAMPLINGCOUNT
sardodirectorysize : $SARDODIRSIZE
tarfilesize        : $TARFILESIZE
time info          : $stime $STIME $etime $ETIME
" >> $WORK

ONETIMEUNIT=$ONETIMETOTALSIZE
if [ $PMAPDIRSIZE -eq 0 ]; then
	PMAPUNIT=0
else
	PMAPUNIT=`$USRBIN/echo $PMAPDIRSIZE |
	$USRBIN/awk '{ printf"%.0f\n",$1/'"$PMAPFILECOUNT"' }'`
fi
# COR-027 start.
DATAUNIT=`$USRBIN/echo $DATADIRSIZE |
$USRBIN/awk '{ printf"%.0f\n",$1/'" $SAMPLINGCOUNT"' }'`
if [ $DATAUNIT -eq 0 ]; then
	DATAUNIT=1
fi
SADCCOUNT=`$USRBIN/echo $SAMPLINGTIME |
$USRBIN/awk '{ printf"%.0f\n",$1/'"$BASEINTERVAL"' }'`
SADCUNIT=`$USRBIN/echo $SADCDIRSIZE |
$USRBIN/awk '{ printf"%.0f\n",$1/'"$SADCCOUNT"' }'`
if [ $SADCUNIT -eq 0 ]; then
	SADCUNIT=1
fi
PACCTUNIT=$PACCOUNTSIZE
# COR-027 end.

$USRBIN/echo "\
dataunit           : $DATAUNIT  X  $SAMPLINGCOUNT
onetimeunit        : $ONETIMEUNIT  X  1
pacctunit          : $PACCTUNIT  X  n
pmapunit           : $PMAPUNIT  X  n
sadcunit           : $SADCUNIT  X  $SADCCOUNT
" >> $WORK
 
$USRBIN/echo "$intervaltimelist" |
$USRBIN/awk 'BEGIN {
printf"  sampling time     sampling interval (sec)\n"
header=sprintf("  sec  min hour ") }
{ header=sprintf("%9s %9s ",header,$1) }
END {	printf"%s\n",header
	printf"=====+====+===+===========+==========+==========+==========+==="
	printf"=======+==========+==========+==========\n" }' >> $WORK

for samplingtime in `$USRBIN/echo "$samplingtimelist"`
do
	$USRBIN/echo $samplingtime |
	$USRBIN/awk '{	printf"%5.0f %4.0f %3.0f ",$1,$1/60,$1/3600 }' >> $WORK
	for intervaltime in `$USRBIN/echo "$intervaltimelist"`
	do
		DATACOUNT=`$USRBIN/echo "" |
		$USRBIN/awk '{	count = '"$samplingtime"' / '"$intervaltime"'
				printf"%.0f\n",count }'`

		SADCCOUNT=`$USRBIN/echo "" |
		$USRBIN/awk '{	count = '"$samplingtime"' / '"$BASEINTERVAL"'
				printf"%.0f\n",count }'`

		PACTCOUNT=`$USRBIN/echo "" |
		$USRBIN/awk '{	count = '"$samplingtime"' / '"$SAMPLINGTIME"'
				printf"%.0f\n",count }'`

		if [ $PMAPFILECOUNT -ne 0 ]; then
			PMAPCOUNT=`$USRBIN/echo "" |
			$USRBIN/awk '{	count = '"$samplingtime"' / '"$SAMPLINGTIME"'
					count = count * '"$PMAPFILECOUNT"'
					printf"%.0f\n",count }'`
		else
			PMAPCOUNT=0
		fi

		$USRBIN/echo $ONETIMEUNIT |
		$USRBIN/awk 'BEGIN {	pmapcount='"$PMAPCOUNT"'
					pmapunit='"$PMAPUNIT"' }
		{
		if ( '"$PMAPCOUNT"' == 0 ) {
			pmapsize = 0
			}
		else {
			pmapsize =  pmapcount * pmapunit
		}
		datasize = '"$DATACOUNT"' * '"$DATAUNIT"'
		pactsize = '"$PACTCOUNT"' * '"$PACCTUNIT"'
		sadcsize = '"$SADCCOUNT"' * '"$SADCUNIT"'
		sardosize = $1 + datasize + sadcsize + pactsize + pmapsize
		printf"%d\n",sardosize >"sardosize_info"
		totalsize = sardosize * 2 + sardosize * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )
		printf" %10.0f",totalsize }' >> $WORK
	done
	$USRBIN/echo "" >> $WORK
done
$USRBIN/echo "" >> $WORK

ONEDAYMAXSIZE=`$USRBIN/awk '{	if ( $1 == "86400" ) {
					printf"%d\n",$NF
					exit
					}
			}' $WORK`

HOLDARCHIVES=`$USRBIN/cat sardosize_info |
$USRBIN/awk '{
printf"%d\n",( '"$AVAILBLOCK"' - '"$ONEDAYMAXSIZE"' ) / ( $1 * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )) }'`

CYCLE=`$USRBIN/echo $ONEDAYMAXSIZE |
$USRBIN/awk '{ printf"%d\n",'"$AVAILBLOCK"' / $1 }'`

$USRBIN/cat sardosize_info | $USRBIN/awk '{
datasize = '"$DATACOUNT"' * '"$DATAUNIT"'
pactsize = '"$PACTCOUNT"' * '"$PACCTUNIT"'
pmapsize = '"$PMAPCOUNT"' * '"$PMAPUNIT"'
sadcsize = '"$SADCCOUNT"' * '"$SADCUNIT"'
printf"the largest blocks :\n"
printf"datadirectory      : %d\n",datasize
printf"onetimedirectories : %d\n",'"$ONETIMEUNIT"'
printf"pacctdirectory     : %d\n",pactsize
printf"pmapdirectory      : %d\n",pmapsize
printf"sadcdirectory      : %d\n",sadcsize
printf"sardodirectory     : %d\n\n",'"$ONETIMEUNIT"'+pmapsize+datasize+sadcsize+pactsize
printf"packedarchivefile  : %d\n",$1 * ( '"$ARCHIVESIZE"' / '"$TARFILESIZE"' )
printf"\n"
}' >> $WORK

$USRBIN/echo "\
INFO:  $ESDIR can hold [ $HOLDARCHIVES ] packed tar archive files, 
INFO:  during the sardo is gathering the latest one of data.
INFO:  sardo can gather maximum for [ $CYCLE ] days data in the directory.

END REPORT" >> $WORK

$USRBIN/mv $WORK ${ESDIR}/sarstats_${ESHOST}_${ESID}/estimate

$USRBIN/rm sardosize_info
}


#
# spstart: start sardo.
#
spstart() {
EXECMODE=$1
if [ "${MODE}_x" = "_x" ]; then
	EXECMODE=NOAUTO
fi
if [ ! -f $PARAMETER ]; then
	LogRecord "ERROR: spstart instance $INSTANCE sardo.param wasn't found."
	ClearfAndExit 1
fi
if [ -f ${SARDODIR}/pidlist ]; then
	LogRecord "ERROR: instance $INSTANCE has already been started."
	ClearfAndExit 1
fi
$USRBIN/date '+%Y.%m.%d' > ${SARDODIR}/date.info
SECONDS=`$USRBIN/awk '{ printf"%s\n",$1 }' $PARAMETER`
STARTTIME=`$USRBIN/awk '{ printf"%s\n",$2 }' $PARAMETER`
STOPTIME=`$USRBIN/awk '{ printf"%s\n",$3 }' $PARAMETER`
SetTimeParameters $SECONDS $STARTTIME $STOPTIME
export INTERVAL COUNT ctime CTIME stime STIME etime ETIME SAMPLING
LogRecord "INFO:  sampling interval  : $INTERVAL"
LogRecord "INFO:  sampling count     : $COUNT"
LogRecord "INFO:  current time       : $ctime $CTIME"
LogRecord "INFO:  start and end time : $stime $STIME $etime $ETIME"
LogRecord "INFO:  sampling mode      : $SAMPLING"
if [ $ctime -gt $etime ]; then
	LogRecord "ERROR: the current time greater than the end time."
	ClearfAndExit 1
fi
if [ $stime -gt $ctime ]; then
	waitsec=`$USRBIN/expr $stime - $ctime`
	LogRecord "INFO:  awaiting start $waitsec sec."
	$USRBIN/sleep $waitsec
else
	waitsec=0
fi
LogRecord "INFO:  sardo will be started with $INTERVAL sec interval."
CheckPacct
if [ $PACCT = "RUN" ]; then
	if [ ! -d $SARDODIR/pacct ]; then
		$USRBIN/mkdir $SARDODIR/pacct
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: process accounting directory couldn't be made."
			ClearfAndExit 1
		fi
	fi
	$USRLIBACCT/accton $SARDODIR/pacct/pacct_info
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: starting process account was failed."
		ClearfAndExit 1
	else
		$USRBIN/echo $$ > ${SARDODIR}/pacctpid
		LogRecord "INFO:  process accounting has been started."
	fi
else
	LogRecord "INFO:  process accounting has already been started."
fi
$USRBIN/echo "$ONETIMECMD" > $TEMPFILE	; DoCommands begin
$USRBIN/echo "$DURCMD"     > $TEMPFILE	; DoDurationCmds data
$USRBIN/echo "$NDCMD"      > $TEMPFILE	; DoNonDurationCmds data
$USRBIN/echo "$SADCCMD"    > $TEMPFILE	; DoSaDcCommand sadc
whichcmd $EVENTCHECK
if [ $COMMANDPATH != "NOTFOUND" ]; then
	if [ -x $COMMANDPATH ]; then
		while [ -f $SARDODIR/go ]
		do
			$COMMANDPATH &					#COR-014
			$USRBIN/sleep $INTERVAL
		done &
		# COR-014 start.
		RETVAL=$?
		EVENTPID=$!
		EVENTCMD=@path
	else
		EVENTCMD=NONE
		# COR-014 end.
	fi
else
	if [ -f ./$EVENTCHECK ]; then
		if [ -x ./$EVENTCHECK ]; then
			while [ -f $SARDODIR/go ]
			do
				./$EVENTCHECK &				#COR-014
				$USRBIN/sleep $INTERVAL
			done &
		# COR-014 start.
			RETVAL=$?
			EVENTPID=$!
			EVENTCMD=@pwd
		else
			EVENTCMD=NONE
		fi
	else
		# COR-016 start.
		if [ -f $EXEC/$EVENTCHECK ]; then
			if [ -x $EXEC/$EVENTCHECK ]; then
				while [ -f $SARDODIR/go ]
				do
					$EXEC/$EVENTCHECK &
					$USRBIN/sleep $INTERVAL
				done &
				RETVAL=$?
				EVENTPID=$!
				EVENTCMD=@any
			else
				EVENTCMD=NONE
			fi
		# COR-014 end.
		# COR-022 start.
		else
			EVENTCMD=NONE
		# COR-022 end.
		fi
		# COR-016 end.
	fi
fi

# COR-014 start.
if [ $EVENTCMD != "NONE" ]; then
	if [ $RETVAL -ne 0 ]; then
		LogRecord "NOTE:  $EVENTCHECK $EVENTCMD couldn't be running."
	else
		LogRecord "INFO:  $EVENTCHECK $EVENTCMD was successfully scheduled."
		$USRBIN/echo $EVENTPID >> $SARDODIR/pidlist
	fi
fi
# COR-014 end.

SadcExecTest
if [ $RETVAL != "OK" ]; then
	LogRecord "ERROR: sadc process pid [ $SADCPID ] wasn't running."
	$USRBIN/echo "ERROR: sadc process pid [ $SADCPID ] wasn't running."
	ClearfAndExit 1
fi
SarDoTermProc $SADCPID
if [ $RETVAL -ne 0 ]; then
	LogRecord "ERROR: the termination process starting was failed."
	ClearfAndExit 1
else
	LogRecord "INFO:  termproc pid [ $PID ] was started successfully."
fi
}


#
# spstop: stop sardo.
#
spstop() {
PID=$$
arg1=$1
if [ "${arg1}_x" = "_x" ]; then
	MODE=NORMAL
else
	if [ $arg1 = "force" ]; then
		MODE=FORCE
	else
		LogRecord "NOTE:  spstop invalid mode."
		MODE=NORMAL
	fi
fi
CheckPacct								#COR-021
if [ -f $SARDODIR/go ]; then
	$USRBIN/rm $SARDODIR/go
fi
if [ -f $SARDODIR/termprocpid ] && [ $MODE = "FORCE" ] ; then
	pid=`$USRBIN/cat $SARDODIR/termprocpid`
	PPID=`$USRBIN/ps -e -o pid,ppid,args |
	$USRBIN/awk '{
	if ( $2 == '"$pid"' ) {
		n=split($3,array,"/")
		if ( array[n] == "sleep")
			found="YES"
		processid=$1
		exit
		}
	} END {	if ( found == "YES" )
			printf"%d\n",processid
		else
			printf"NOPPID\n"
	}'`
	$USRBIN/kill -9 $pid 1>/dev/null 2>&1
	$USRBIN/kill -9 $PPID 1>/dev/null 2>&1
	$USRBIN/rm $SARDODIR/termprocpid
fi
if [ -f $SARDODIR/pidlist ]; then
	for pid in `$USRBIN/cat $SARDODIR/pidlist`
	do
		RETVAL=ALIVE
		while [ $RETVAL != "NOTALIVE" ]
		do
			RETVAL=`$USRBIN/ps -e -o pid= |
			$USRBIN/awk '{	if ( $1 == '"$pid"' ) {
						found="YES"
						pid=$1
						exit
						}
				}
			END {	if ( found == "YES" )
					printf"%d\n",pid
				else
					printf"NOTALIVE\n" }'`
			if [ $RETVAL != "NOTALIVE" ]; then
				PPID=`$USRBIN/ps -e -o pid,ppid,args |
				$USRBIN/awk '{
				if ( $2 == '"$pid"' ) {
					n=split($3,array,"/")
					if ( array[n] == "sleep")
						found="YES"
					processid=$1
					exit
					}
				} END {	if ( found == "YES" )
						printf"%d\n",processid
					else
						printf"NOPPID\n"
				}'`
				if [ $PPID != "NOPPID" ]; then
					$USRBIN/kill -9 $PPID 1>/dev/null 2>&1
				fi
				$USRBIN/kill -9 $pid 1>/dev/null 2>&1
				$USRBIN/sleep 1
			fi
			PrintDot
		done
	done
	BlankLine
	$USRBIN/rm $SARDODIR/pidlist
fi
$USRBIN/echo "$ONETIMECMD" > $TEMPFILE	; DoCommands end
if [ -f ${SARDODIR}/pacctpid ]; then
	$USRLIBACCT/accton
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: stop process accounting was faild."
		ClearfAndExit 1
	else
		LogRecord "INFO:  process accounting was stopped."
	fi
	$USRBIN/rm ${SARDODIR}/pacctpid
else
	# COR-021 start.
	if [ -f ${SARDODIR}/pacct/pacct_info ]; then
		$USRLIBACCT/accton
	else
		LogRecord "INFO:  process accounting wasn't stopped."
	fi
	# COR-021 end.
fi
if [ ! -d $SARDODIR/misc ]; then
	$USRBIN/mkdir $SARDODIR/misc
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making misc directory was failed."
		ClearfAndExit 1
	fi
fi
for i in `$USRBIN/echo "$GATHERFILES"`
do
	if [ -f $i ]; then
		$USRBIN/cp $i $SARDODIR/misc/.
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: file $i copy was failed."
			ClearfAndExit 1
		fi
	fi
done
spdevinfo
cd $SARDODIR
for i in `$USRBIN/find * -print | $USRBIN/grep -v tempfile`
do
	if [ -f $i ]; then
		$USRBIN/wc -C $i
		if [ $? -ne 0 ]; then
			LogRecord "ERROR: wc command was failed."
			ClearfAndExit 1
		fi
	fi
done > $TEMPFILE
$USRBIN/mv $TEMPFILE $SARDODIR/misc/fsize_info
if [ $? -ne 0 ]; then
	LogRecord "ERROR: moving fsize_info file was failed."
	ClearfAndExit 1
fi
# COR-025 start.
UNAME_I=`$USRBIN/uname -i`
if [ $UNAME_I = "i86pc" ]; then
	$USRBIN/echo "$MISCI86CMD" > $TEMPFILE	; DoCommands misc
else
	$USRBIN/echo "$MISCCMD" > $TEMPFILE	; DoCommands misc
fi
# COR-025 end.
}


#
# Archive files.
#
sparchive() {
if [ -f $SARDODIR/pidlist ]; then
	LogRecord "ERROR: sardo instance $INSTANCE is still running."
	ClearfAndExit 1
fi
cd $LOGDIR
DIR=`$USRBIN/basename $SARDODIR`
DATE=`$USRBIN/date '+%y%m%d%H%M%S'`
$USRSBIN/tar -cvf ${SARDODIR}_${DATE}.tar $DIR
if [ $? -ne 0 ]; then
	LogRecord "ERROR: tar command was failed."
	ClearfAndExit 1
fi
$USRBIN/gzip ${SARDODIR}_${DATE}.tar					#COR-004
if [ $? -ne 0 ]; then
	LogRecord "ERROR: gzip command was failed."			#COR-004
	ClearfAndExit 1
fi
ROOTDIR=/
TESTDIR=$EXECDIRECTORY
FOUND=NO
while [ "$ROOTDIR" != "$TESTDIR" ]
do
	TESTDIR=`$USRBIN/dirname $TESTDIR`
	if [ $TESTDIR = $LOGDIR ]; then
		FOUND=YES
		ROOTDIR=/
	fi
done
if [ $FOUND = "YES" ]; then
	$USRBIN/echo "NOTE:  Although tar archive and gzip were successfully completed," #COR-004
	$USRBIN/echo "NOTE:  the working directory $EXECDIRECTORY is below $LOGDIR."
	$USRBIN/echo "NOTE:  $DIR directory won't be cleared."
else
	$USRBIN/rm -rf $SARDODIR
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: remove $SARDODIR was failed."
		ClearfAndExit 1
	fi
	$USRBIN/echo "INFO:  tar archive and compress were successfully completed."
fi
}


#
# Collect pmap -x information.
#
spexpmap() {
if [ ! -d $SARDODIR/pmap ]; then
	$USRBIN/mkdir $SARDODIR/pmap
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making pmap directory was failed."
		ClearfAndExit 1
	fi
fi
DATE=`$USRBIN/date '+%y%m%d%H%M%S'`
for pid in `$USRBIN/ps -e -o pid=`
do
	$USRPROCBIN/pmap -x $pid 2>/dev/null
done > $SARDODIR/pmap/pmap_x_${DATE}_info
#if [ $? -ne 0 ]; then
#	LogRecord "ERROR: pmap -x was failed."
#	ClearfAndExit 1
#fi
LogRecord "INFO:  an intermediate pmap -x command was completed."
}


#
# COR-026 start.
#
# Collect memstat information.
#
spexmemstat() {
if [ ! -d $SARDODIR/memstat ]; then
	$USRBIN/mkdir $SARDODIR/memstat
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making memstat directory was failed."
		ClearfAndExit 1
	fi
fi
DATE=`$USRBIN/date '+%y%m%d%H%M%S'`
if [ -f $USRBIN/mdb ]; then
	if [ -x $USRBIN/mdb ]; then
		$USRBIN/mdb -k -y > $SARDODIR/memstat/memstat_${DATE}_info 2>/dev/null <<-EOF	#COR-028
		::memstat
		::quit
		EOF
		RETVAL=$?
		if [ $RETVAL -ne 0 ]; then
			LogRecord "ERROR: an intermediate mdb ::memstat command was failed."
			ClearfAndExit 1
		fi
		LogRecord "INFO:  an intermediate mdb ::memstat command was completed."
	else
		LogRecord "NOTE:  $USRBIN/mdb command isn't executable."
	fi
else
	LogRecord "NOTE:  $USRBIN/mdb command not found."
fi
}
# COR-026 end.


#
# List disk address information.
#
spdevinfo() {
if [ ! -d $SARDODIR/misc ]; then
	$USRBIN/mkdir $SARDODIR/misc
	if [ $? -ne 0 ]; then
		LogRecord "ERROR: making misc directory was failed."
		ClearfAndExit 1
	fi
fi
for dir in dsk
do
	for dev in `$USRBIN/ls /dev/$dir/*`
	do
		$USRBIN/ls -l $dev |
		$USRBIN/nawk 'BEGIN {
			while (( getline <"/etc/path_to_inst") > 0 ) {
				split($1,array,"\"")
				tag=sprintf("../../devices%s",array[2])
				physicalname[tag]=tag
				instance[tag]=$2
				split($3,array,"\"")
				bindingname[tag]=array[2]
			}
		}
		{
			n=split($9,device,"/")
			devname=device[n]
			split($NF,array,":")
			if ( array[1] == physicalname[array[1]] ) {
				printf"%s ",devname
				dev=sprintf("%s",bindingname[array[1]])
				dev=sprintf("%s%s",dev,instance[array[1]])
				dev=sprintf("%s,%s",dev,array[2])
				printf"%-8s ",dev
				n=split(array[1],devinfo,"/")
				split(devinfo[n],info,"@")
				printf"%s ",info[2]
				n--
				do {
					split(devinfo[n],info,"@")
					printf"%s %s   ",info[1],info[2]
					n--
				} while ( n > 3 )
				printf"\n"
			}
		}'
	done
done > $SARDODIR/misc/device_info
if [ $? -ne 0 ]; then
	LogRecord "ERROR: making device info file was failed."
	# COR-029 ClearfAndExit 1
else
	LogRecord "INFO:  list devinfo was successfully completed."
fi
}


#
# Unpack sarstats data into the sardo directory.
#
spunpack () {
WORK=$LOGDIR/workfile$$
cd $LOGDIR
$USRBIN/ls  sarstats_*_${INSTANCE}_*.tar.gz 2>/dev/null > $WORK		#COR-004
if [ ! -s $WORK ]; then
	$USRBIN/echo "ERROR: no any unpack files were found."
	ClearfAndExit 1
fi
N=`$USRBIN/wc -l $WORK | $USRBIN/awk '{ printf $1 }'`
if [ $N -gt 1 ]; then
	if [ $BATCHMODE = "YES" ]; then
		$USRBIN/echo "NOTE:  any unpack files list couldn't be displayed."
		ClearfAndExit 1
	fi
	input="NO"
	while [ $input = "NO" ]
	do
		BlankLine
		# COR-004 start.
		$USRBIN/ls -l sarstats_*_${INSTANCE}_*.tar.gz |
		# COR-004 end.
		$USRBIN/awk '{
		printf"%s  %3s %2s %s %9d\n",$9,$6,$7,$8,$5}' > $WORK
		$USRBIN/awk '{ printf"%10d.  %s\n",NR,$0 }' $WORK
		$USRBIN/echo "         Several unpack files found.\n"
		$USRBIN/echo "         Select file number ? : \c"
		read INPUT
		if [ "${INPUT}_x" != "_x" ]; then
			if [ ${INPUT} -le ${N} ] && [ ${INPUT} != 0 ]; then
				input="YES"
				FILENAME=`$USRBIN/awk '
				{	if ( NR == "'$INPUT'" )
						printf"%s\n",$1 }' $WORK`
			fi
		else
			input="NO"
		fi
	done
	$USRBIN/echo "INFO:  the unpack file $FILENAME was chosen."
else
	FILENAME=`$USRBIN/cat $WORK`
	$USRBIN/echo "NOTE:  the unpack file $FILENAME was assumed."
fi
if [ -s $WORK ]; then
	$USRBIN/rm $WORK
fi
FILENAME=`$USRBIN/echo $FILENAME |
$USRBIN/awk '{	split($1,array,".")
		printf"%s.tar\n",array[1],array[2] }'`
DIRECTORYNAME=`$USRBIN/echo $FILENAME |
$USRBIN/awk '{	split($1,array,".")
		split(array[1],dir,"_")
		printf"%s_%s_%s\n",dir[1],dir[2],dir[3] }'`
if [ -d $DIRECTORYNAME ]; then
	$USRBIN/echo "ERROR: directory $DIRECTORYNAME already exists."
	ClearfAndExit 1
fi
$USRBIN/gunzip -d ${FILENAME}.gz 2>/dev/null				#COR-004
if [ $? -ne 0 ]; then
	$USRBIN/echo "ERROR: gunzip command was failed."		#COR-004
	ClearfAndExit 1
fi
$USRBIN/tar -xvf $FILENAME
if [ $? -ne 0 ]; then
	$USRBIN/echo "ERROR: tar -xvf command was failed."
	ClearfAndExit 1
fi
$USRBIN/echo "INFO:  spunpack was successfully completed."
}


############################################
#         Main routine start here.         #
############################################

#
# Set arguments.
#
EXECSHELL=$0
numberofargs=$#
execshell=`$USRBIN/basename $EXECSHELL`
FUNCTION=$1; ARG2=$2; ARG3=$3; ARG4=$4
LANG=C; export LANG

if [ $numberofargs = 0 ]; then
	$USRBIN/echo "ERROR: argument missing."
	exit 1
fi


#
# Determine the LOGDIR environment variable.
#
if [ "${LOGDIR}_x" = "_x" ]; then
	LOGDIR=/var/tmp
fi
	

#
# Get the OS name.
#
OS=`$USRBIN/uname -a |
$USRBIN/awk '{ printf"%s\n",substr($1,1,6) }'`

#
# Get the working directory and the actual sardo.sh path name.
#
EXECDIRECTORY=`$USRBIN/pwd`
whichcmd $execshell
RESULT=$COMMANDPATH
if [ $RESULT = "NOTFOUND" ]; then
	ISDOT=`$USRBIN/echo $EXECSHELL | $USRBIN/awk '{
	if ( substr($1,1,1) == "." )
		printf"YES\n"
	else
		printf"NO\n"
	}'`
	if [ $ISDOT = "YES" ]; then
		SARDOEXECSHELL=${EXECDIRECTORY}/${execshell}
	else
		SARDOEXECSHELL=$EXECSHELL
	fi
else
	SARDOEXECSHELL=$COMMANDPATH
fi

# COR-014 start.
EXEC=`$USRBIN/dirname $SARDOEXECSHELL`
export EXEC
# COR-014 end.

#
# Check root user uid.
#
UidCheck

#
# Set INSTANCE and the other environment variables.
#
SetInstance

#
# Set BATCHMODE environment variable.
#
SetBatchMode

#
# Check and make SARDO directory under the workdirectory $LOGDIR.
#
SarDoDirectory

#
# Go to the individual function.
#
case $FUNCTION in
auto)		spautoexec	$ARG2 $ARG3 $ARG4	;;
devinfo)	spdevinfo				;;
estimate)	spestimate	$ARG2 $ARG3 $ARG4	;;
memstat)	spexmemstat				;;		#COR-026
pmap)		spexpmap				;;
setup)		spsetup		$ARG2 $ARG3 $ARG4	;;
start)		spstart					;;
stop)		spstop		$ARG2			;;
tar)		sparchive				;;
unpack)		spunpack				;;
*)		LogRecord "ERROR: argument missing. [ $FUNCTION ]"
		ClearfAndExit 1				;;
esac

#
# clear work files and exit.
#
ClearfAndExit

#
# end script.
